"""
Base sklearn based classifiers
"""

import warnings
from sklearn.svm import SVC
from sklearn.neural_network import MLPClassifier
from sklearn.gaussian_process import GaussianProcessClassifier
from sklearn.ensemble import RandomForestClassifier, AdaBoostClassifier
from sklearn.ensemble import ExtraTreesClassifier
from sklearn.ensemble import GradientBoostingClassifier, VotingClassifier
from sklearn.naive_bayes import MultinomialNB, GaussianNB
from sklearn.discriminant_analysis import QuadraticDiscriminantAnalysis

warnings.simplefilter("ignore")


OPTIONS = {'RandomForestClassifier': RandomForestClassifier,
           'MultinomialNB': MultinomialNB,
           'GaussianNB': GaussianNB,
           'SVC': SVC,
           'MLPClassifier': MLPClassifier,
           'AdaBoostClassifier': AdaBoostClassifier,
           'QuadraticDiscriminantAnalysis': QuadraticDiscriminantAnalysis,
           'GaussianProcessClassifier': GaussianProcessClassifier,
           'ExtraTreesClassifier': ExtraTreesClassifier,
           'GradientBoostingClassifier': GradientBoostingClassifier,
           'VotingClassifier': VotingClassifier
           }


class BaseSklearn:
    """Base Sklearn classifier"""

    def __new__(cls, name, params=None):

        if name not in OPTIONS.keys():
            raise NameError(
                f"please select one of these as the name: {[x for x in OPTIONS.keys()]}")

        if params:
            clf = OPTIONS.get(name)(**params)
            print(
                f"""classification model configured to use {clf.__class__.__name__} algorithm with parameters:\n{params}""")

        else:
            clf = OPTIONS.get(name)()
            print(
                f"""classification model configured to use {clf.__class__.__name__} algorithm.\nnote: running with default configuration""")
        return clf



"""

Base pytorch based deep classifiers

"""
import torch
from .yoonkim import YoonKimCnn


OPTIONS = {"yoonkim": YoonKimCnn}


class BaseTorch:
    def __new__(cls, name, params=None):

        if name not in OPTIONS.keys():
            raise NameError(
                f"please select one of these as the name: {[x for x in OPTIONS.keys()]}")

        if params:
            clf = OPTIONS.get(name)(**params)
            print(
                f"""classification model configured to use {clf.__class__.__name__} algorithm with parameters:\n{params}""")

        else:
            clf = OPTIONS.get(name)()
            print(
                f"""classification model configured to use {clf.__class__.__name__} algorithm.\nnote: running with default configuration""")
        return clf



import torch
import torch.nn as nn
import torch.nn.functional as F
from torch.autograd import Variable


class YoonKimCnn(nn.Module):
    """
    sentence classification model  based on https://arxiv.org/abs/1408.5882 by Yoon Kim.
    """

    def __init__(self, vocabulary_size, embedding_size,
                 channel_out, filter_heights, dropout=0.5):
        """
        parameters: 
        __________

        vocabulary_size: int
            size of the vocabulary being used

        embedding_size: int
            size of the embeddings to be used.

        channel_out: int
            number of output channels

        filter_heights: tuple
            the heights of the convolution filters to be used (ex: (2, 3, 4)) 

        dropout: float
            dropout size

        returns: 
        _______

        YoonKimCnn torch model

        """
        super().__init__()
        self.embedding = nn.Embedding(vocabulary_size, embedding_size)
        self.conv = nn.ModuleList(
            [nn.Conv2d(1, channel_out, (fh, embedding_size)) for fh in filter_heights])
        self.dropout = nn.Dropout(dropout)
        self.fc1 = nn.Linear(channel_out*len(filter_heights), 1)

    def forward(self, x):
        x = self.embedding(x)
        x = x.unsqueeze(1)
        x = [F.relu(conv(x)).squeeze(3) for conv in self.conv]
        x = [F.max_pool1d(i, i.size(2)).squeeze(2) for i in x]
        x = torch.cat(x, 1)
        x = self.dropout(x)
        x = self.fc1(x)
        prob = F.sigmoid(x)
        return prob



from .sklearn.base_sklearn import BaseSklearn
from .torch.base_torch import BaseTorch


class Texion:
    def __new__(cls, mode, name, params=None):
        if mode == "Sklearn":
            print(f"configured to run with {mode}")
            return BaseSklearn(name, params)
        if mode == "Torch":
            print(f"configured to run with {mode}")
            return BaseTorch(name, params)



from sklearn.model_selection import train_test_split


def train_test_split_data(texts, labels, test_size=0.2, **kwargs):
    train_texts, test_texts, train_labels, test_labels = train_test_split(
        texts, labels, test_size=test_size, **kwargs)
    return train_texts, test_texts, train_labels, test_labels
